"""
Copyright (C) 2021  Alexander "PapaTutuWawa"

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
import asyncio
from functools import partial
import logging

from mira.storage import StorageManager
from mira.subscription import SubscriptionManager

logger = logging.getLogger("mira.module")


class ManagerWrapper:
    """
    Wrapper class around {Storage, Subscription}Manager in order
    to to expose those directly to the modules without allowing them
    access to other modules.
    """

    _name = ""
    _manager = None

    def __init__(self, name, manager):
        self._name = name
        self._manager = manager.get_instance()

    def __getattr__(self, key):
        if not key in dir(self._manager):
            raise AttributeError(
                "Attribute %s does not exist in wrapped"
                " class %s" % (key, type(self._manager))
            )

        return partial(getattr(self._manager, key), self._name)


class BaseModule:
    def __init__(self, base, config={}, subcommand_table={}, name=""):
        self._name = name
        self._base = base
        self._config = config
        self._subcommand_table = subcommand_table
        self._local_only = self.get_option("restrict_local", False)
        self._restricted = (
            "restrict_local" in self._config or "allowed_domains" in self._config
        )

        self._stm = ManagerWrapper(self._name, StorageManager)
        self._sum = ManagerWrapper(self._name, SubscriptionManager)

        logger.debug("Init of %s done" % (self._name))

    def get_option(self, key, default=None):
        """
        Like dict.get(), but for the options from the bot configuration
        file.

        If key does not exist, then default will be returned.
        """
        return self._config.get(key, default)

    def send_message(self, to, body):
        """
        A simple wrapper that sends a message with type='chat' to
        @to with @body as the body
        """
        self._base.send_message_wrapper(to, body)

    def _base_on_command(self, cmd, msg):
        def run(func):
            loop = asyncio.get_event_loop()
            loop.create_task(func(cmd, msg))

        logger.debug("Received command: %s" % (str(cmd)))

        if not self._subcommand_table:
            run(self.on_command)
        elif cmd and cmd[0] in self._subcommand_table:
            run(self._subcommand_table[cmd[0]])
        else:
            if "*" in self._subcommand_table:
                run(self._subcommand_table["*"])
